<?php
/**
 * @package     CSVI
 * @subpackage  Users
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - @year@ RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

use Joomla\CMS\Component\ComponentHelper;
use Joomla\Database\DatabaseDriver;

defined('_JEXEC') or die;

/**
 * Users maintenance.
 *
 * @package     CSVI
 * @subpackage  Users
 * @since       7.0
 */
#[AllowDynamicProperties]
class Com_usersMaintenance
{
	/**
	 * Database connector
	 *
	 * @var    DatabaseDriver
	 * @since  7.0
	 */
	private $db = null;

	/**
	 * Logger helper
	 *
	 * @var    CsviHelperLog
	 * @since  7.0
	 */
	private $log = null;

	/**
	 * CSVI Helper.
	 *
	 * @var    CsviHelperCsvi
	 * @since  7.0
	 */
	private $csvihelper = null;

	/**
	 * Set if we are running in CLI
	 *
	 * @var    bool
	 * @since  7.2.0
	 */
	private $isCli = false;

	/**
	 * Constructor.
	 *
	 * @param   JDatabaseDriver  $db          The database class
	 * @param   CsviHelperLog    $log         The CSVI logger
	 * @param   CsviHelperCsvi   $csvihelper  The CSVI helper
	 * @param   bool             $isCli       Set if we are running CLI mode
	 *
	 * @since   7.0
	 */
	public function __construct($db, CsviHelperLog $log, CsviHelperCsvi $csvihelper, bool $isCli = false)
	{
		$this->db         = $db;
		$this->log        = $log;
		$this->csvihelper = $csvihelper;
		$this->isCli      = $isCli;
	}

	/**
	 * Update Custom available fields that require extra processing.
	 *
	 * @return  void.
	 *
	 * @since   7.2.0
	 */
	public function customAvailableFields()
	{
		if (ComponentHelper::isEnabled('com_fields'))
		{
			// Update Joomla custom fields
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName(['name', 'type']))
				->from($this->db->quoteName('#__fields'))
				->where($this->db->quoteName('state') . ' = 1')
				->where($this->db->quoteName('context') . ' = ' . $this->db->quote('com_users.user'));
			$this->db->setQuery($query);

			$customFields = $this->db->loadObjectList();

			if ($customFields)
			{
				$query->clear()
					->insert($this->db->quoteName('#__csvi_availablefields'))
					->columns($this->db->quoteName(['csvi_name', 'component_name', 'component_table', 'component', 'action']));

				/** @var stdClass $customField */
				foreach ($customFields as $customField)
				{
					$query->values(
						$this->db->quote($customField->name) . ',' .
						$this->db->quote($customField->name) . ',' .
						$this->db->quote('user') . ',' .
						$this->db->quote('com_users') . ',' .
						$this->db->quote('import')
					);
					$query->values(
						$this->db->quote($customField->name) . ',' .
						$this->db->quote($customField->name) . ',' .
						$this->db->quote('user') . ',' .
						$this->db->quote('com_users') . ',' .
						$this->db->quote('export')
					);

					if ($customField->type === 'subform')
					{
						$query->values(
							$this->db->quote($customField->name . '_raw') . ',' .
							$this->db->quote($customField->name . '_raw') . ',' .
							$this->db->quote('user') . ',' .
							$this->db->quote('com_users') . ',' .
							$this->db->quote('import')
						);
						$query->values(
							$this->db->quote($customField->name . '_raw') . ',' .
							$this->db->quote($customField->name . '_raw') . ',' .
							$this->db->quote('user') . ',' .
							$this->db->quote('com_users') . ',' .
							$this->db->quote('export')
						);
					}
				}

				$this->db->setQuery($query)->execute();
			}
		}
	}

	/**
	 * Threshold available fields for extension
	 *
	 * @return  int Hardcoded available fields
	 *
	 * @since   7.0
	 */
	public function availableFieldsThresholdLimit()
	{
		return 33;
	}
}
